<?php
/**
*
* @package hjw calendar Extension
* @copyright (c) 2020 hjw
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace hjw\calendar\event;

/**
* @ignore
*/
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class main_listener implements EventSubscriberInterface
{
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'								=> 'load_language_on_setup',
			'core.posting_modify_template_vars'   			=> 'calendar',
			'core.page_header'     							=> 'calendar_on_header',
			'core.viewonline_overwrite_location'			=> 'viewonline_page',
			'core.submit_post_end'							=> 'send_data_to_table',
			'core.viewtopic_assign_template_vars_before'	=> 'modify_participants_list',
			'core.viewtopic_modify_post_row'				=> 'display_participants_list',
		);
	}

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\request\request_interface */
	protected $request;
	
	/** @var string php file extension */
	protected $php_ext;

	/** @var string phpbb root path */
	protected $phpbb_root_path;
 
	/** @var \phpbb\event\dispatcher_interface */
    protected $phpbb_dispatcher;	
	
	/**
	* Constructor
	*
	* @param \phpbb\config\config		$config
	* @param \phpbb\controller\helper	$helper
	* @param \phpbb\template\template	$template
	* @param \phpbb\user				$this->user
	*/

	public function __construct(\phpbb\config\config $config, \phpbb\db\driver\driver_interface $db, \phpbb\controller\helper $helper, 
								\phpbb\auth\auth $auth, \phpbb\template\template $template, \phpbb\user $user, \phpbb\request\request $request,
								\phpbb\event\dispatcher_interface $phpbb_dispatcher, $phpbb_root_path, $phpEx )
	{
		$this->config = $config;
		$this->db = $db;
		$this->helper = $helper;
		$this->template = $template;
		$this->user = $user;
		$this->auth = $auth;
		$this->request = $request;
		$this->phpex = $phpEx;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->ext_root_path = $phpbb_root_path . 'ext/hjw/calendar/';
        $this->phpbb_dispatcher = $phpbb_dispatcher;
	}

	public function viewonline_page($event)
	{
		switch ($event['on_page'][1])
		{
			case 'app':
				if (strrpos($event['row']['session_page'], '/calendar'))
				{
					$event['location'] = $this->user->lang('VIEWING_CALENDAR');
					$event['location_url'] = $this->helper->route('hjw_calendar_controller');
				}
			break;
		}
	}

	public function load_language_on_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'hjw/calendar',
			'lang_set' => 'calendar',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function display_participants_list($event)
	{ 
		global $str_date;
		$post_id	= $event['row']['post_id'];
		$forum_id	= $event['row']['forum_id'];
		$topic_id	= $event['row']['topic_id'];
		$d			= $this->request->variable('d',0);
		$p			= $this->request->variable('p',0);
		$event_date	= $this->request->variable('event_date',0);

		$event_date = ($d != 0 && $p == $post_id) ? $d : time(); 

		include_once($this->ext_root_path . 'includes/constants' . $this->phpex);

		$p_id = $event['cp_row'];
		
		$sql = 'SELECT *
				FROM ' . CALENDAR_TABLE . '
				WHERE post_id = ' . $post_id;
		$result = $this->db->sql_query($sql);
		$event_row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		if ($event_row)
		{
			$d_f = explode('-', $event_row['date_from'] . '-0-0');
			$cal_date = ($event_row['date_from'] > '1970-01-01') ? date_format ($event_row['date_from'], false) : '';
			$t = explode('-', $event_row['date_to'] . '-0-0');
			if ($t[0] >= $d_f[0] && $t[1] >=$d_f[1] && $t[2] > $d_f[2])
			{
				$cal_date .= ($event_row['date_from'] > '1970-01-01') ? '-<br />' . date_format ($event_row['date_to'], false) : '';
			}
			$pd = strtotime($event_row['date_from']);
			$sql = 'SELECT *
					FROM ' . CALENDAR_EVENT_TABLE . '
					WHERE id = ' . $event_row['event_id'];

			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			$sql = 'SELECT *
					FROM ' . CALENDAR_FORUMS_TABLE . '
					WHERE forum_id = ' . $forum_id;
			$result = $this->db->sql_query($sql);
			$forum_row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);
			if ($forum_row)
			{
				if ($forum_row['allowed'] == 1)
				{
					if($event_row['calendar_repeat'])
					{
						$month = ($event_row['repeat_month_number'] == 1) ? '': $event_row['repeat_month_number'] . '. ';
						if ($event_row['repeat_dm'] == 0)
						{
							$text = $this->user->lang['CALENDAR_REPEAT_MONTH_1'] . $month . $this->user->lang['CALENDAR_REPEAT_MONTH_2'];
							$start = $event_date - 10 * $event_row['repeat_month_number']*30*24*60*60;
							$r_m = $event_row['repeat_month_number'];
							$r_d = 0;
						}
						if ($event_row['repeat_dm'] == 1)
						{
							$text = $this->user->lang['CALENDAR_REPEAT_DAYS_1'] . $event_row['repeat_day_number'] . $this->user->lang['CALENDAR_REPEAT_DAYS_2'];
							$start = $event_date - 10 * $event_row['repeat_day_number']*24*60*60;
							$r_m = 0;
							$r_d = $event_row['repeat_day_number'];
						}
						if ($event_row['repeat_dm'] == 2)
						{
							$text = $event_row['repeat_day_number'] . '. ' . $this->user->lang['datetime'][jddayofweek($event_row['weekday']-1,1)] . ' ' . $this->user->lang['CALENDAR_EVERY_WEEKDAY'] . ' ' . $month . $this->user->lang['CALENDAR_REPEAT_MONTH_2'];
							$start = $event_date - 10 * $event_row['repeat_month_number']*30*24*60*60;
							$r_m = $event_row['repeat_month_number'];
							$r_d = 0;
							$w_d = $event_row['weekday'];
							$r_d_c = $event_row['repeat_day_number']-1;
						}
					}
					if($row['participants'])
					{
						if($event_row['calendar_repeat'])
						{
							$d_t = explode('-', $event_row['date_to']);
							$end = ($event_row['date_to'] != '0000-00-00') ? mktime (0, 0, 0, $d_t[1], $d_t[2], $d_t[0]) : mktime (0, 0, 0, 1, 18, 2038) ;

							$j = 0;
							$i = 0;
							while($i<20)
							{
								$e_d = mktime(0,0,0,$d_f[1]+($j*$r_m),$d_f[2]+($j*$r_d),$d_f[0]);
								if ($event_row['repeat_dm'] == 2)
								{
									$begin_weekday = date("N", mktime(0,0,0,$d_f[1]+($j*$r_m),1,$d_f[0]));
									$wd = $event_row['weekday'] - $begin_weekday + 1;
									$wd = ($wd <= 0) ? $wd + 7 : $wd;
									$wd += 7*$r_d_c;
									$e_d = ($wd > date("t",mktime(0,0,0,$d_f[1]+($j*$r_m),1,$d_f[0]))) ? 0 : mktime(0,0,0,$d_f[1]+($j*$r_m),$wd,$d_f[0]);
								}
								$option = '';
								if ($e_d > $start)
								{
									if ($e_d >= $end)
									{
										break;
									}
									$i++;
									$selected = ($event_date == $e_d) ? ' selected="selected"' : '';
									$selected = ($selected == '' && $e_d > $event_date && $pd == 0) ? ' selected="selected"' : $selected;
									$pd = ($selected) ? $e_d : $pd;
									$number['yes']	= $number['no'] =$number['mb']	= 0;
									$sql = 'SELECT participants, number
											FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
											WHERE post_id = "' . $post_id . '"
												AND event_date = "' . date("Y-m-d",$e_d) . '"
											OR post_id = "' . $post_id . '"
												AND event_date = "' . date("Y-m-d",$e_d) . '"';
									$result = $this->db->sql_query($sql);
									while($part_row = $this->db->sql_fetchrow($result))
									{
										$number[$part_row['participants']] += (int)$part_row['number'];
									}
									$nr = ($number['yes']) ? ' [' . $number['yes'] . ']' : '';
						
									$option .='<option value="' . $e_d . '"' . $selected . '>' . date_format(date("Y-m-d",$e_d) . '-0-0',false) . $nr . '</option>';
									$p_id['row']['EVENT_LIST'][] = array(
										'EVENT_DATE_LIST'			=> $option,
									);
									$p_id['row']['ACTION'] = array(
										'U_ACTION'				=>	'./viewtopic.php?p=' . $post_id . '#p' . $post_id,
									);
								}
								$j++;
							}	
						}
						$sql = 'SELECT *
								FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
								WHERE post_id = "' . $post_id . '"
									AND event_date = "' . date("Y-m-d", $pd) . '"
								OR post_id = "' . $post_id . '"
									AND event_date = ""';
						$result = $this->db->sql_query($sql);
						while($part_row = $this->db->sql_fetchrow($result))
						{
							$sql = 'SELECT user_colour, username, user_timezone, user_id
									FROM ' . USERS_TABLE . '
									WHERE user_id = ' . $part_row['user_id'];
							$user_result = $this->db->sql_query($sql);
							while($user_row = $this->db->sql_fetchrow($user_result))
							{
								$number[$part_row['participants']] += intval($part_row['number']);
								$p_id['row']['LIST'][] = array(
									'PARTICIPANTS_USER'			=> $user_row['username'],
									'PARTICIPANTS_USER_LINK'	=> append_sid($this->phpbb_root_path . 'memberlist.php?mode=viewprofile&amp;u=' . $user_row['user_id']),
									'PARTICIPANTS_COLOUR'		=> $user_row['user_colour'],
									'PARTICIPANTS_NUMBER'		=> $part_row['number'],
									'PARTICIPANTS_PART'			=> $this->user->lang['CALENDAR_' . strtoupper($part_row['participants'])],
									'PARTICIPANTS_COMMENTS'		=> $part_row['comments'],
									'PARTICIPANTS_DATE'			=> date_format($part_row['date'],true),
								);
							}
						}
						$p_id['row']['COUNT'] = array(
							'PARTICIPANTS_COUNT'	=> $number['yes'] . '&nbsp;/&nbsp;' . $number['mb'] . '&nbsp;/&nbsp;' . $number['no'],
						);
					}
					$link = ($pd) ? 'calendar/?month=' . date("m",$pd) . '&amp;year=' . date("Y",$pd) : (($d_f[0] . '-' . $d_f[1]) >= '1970-01') ? 'calendar/?month=' . $d_f[1] . '&amp;year=' . $d_f[0] : 'calendar/?month=' . $d_f[1] . '&amp;year=' . date("Y");
					if(!$this->config['enable_mod_rewrite'])
					{
						$link = 'app.php/' . $link;
					}
					$link = append_sid($link);
					$p_id['row']['CALENDAR_ENTRY'] = array(
						'ENTRY'				=> $cal_date,
						'NAME'				=> $event_row['event_name'],
						'LINK'				=> $link,
						'TEXT'				=> $text,
						'EVENT'				=> $row['event'],
						'COLOR'				=> $row['color'],
						'BCOLOR'			=> $row['bcolor'],
						'BIG'				=> $row['big'],
						'PARTICIPANTS_LIST'	=> $row['participants'],
						'EVENT_DATE'		=> $pd,
						'U_PARTICIPANTS'	=> append_sid($this->phpbb_root_path . 'viewtopic.php?f=' . $forum_id . '&amp;p=' . $post_id . '#p' . $post_id),
					);
					$event['cp_row'] = $p_id;
				}
			}
		}
	}

	public function calendar_on_header($event)
	{ 
		global	$weekday, $wday, $c_event, $c_date, $user, $c_event, $c_c, $c_from, $c_to, $c_name, $c_link, $c_part, $c_nr, 
				$c_title, $c_color, $c_bcolor, $c_big, $c_canceled;
		
		$dc = $this->request->variable('dc','');
		$sid = $this->request->variable('sid','');
		$c_c = -1;
		
		$c_from = array();
		$c_to = array();
		$c_name = array();
		$c_link = array();				
		$c_part = array();
		$c_nr = array();
		$c_title = array();
		$c_color = array();
		$c_bcolor = array();
		$c_big = array();
		$c_canceled = array();
		
		$calendar = true;

		if($this->config['hjw_calendar_on_index_off'])
		{
			$url = $this->request->variable('REQUEST_URI', '', false,\phpbb\request\request_interface::SERVER);
			$cookie = $this->request->variable($this->config['cookie_name'] . '_calendar_on_header', '', false,\phpbb\request\request_interface::COOKIE);
			if($sid)
			{
				$s = '\?sid=' . $sid;
				$url = preg_replace ('/' . $s . '/', '', $url);
				$s = '\&amp;sid=' . $sid;
				$url = preg_replace ('/' . $s . '/', '', $url);
			}	
			$url = preg_replace(array('/\?dc=off/', '/\?dc=on/', '/&amp;dc=off/', '/&amp;dc=on/') , '', $url);
			$i = strpos($url, '?');
			$url .= ($i) ?  '&' : '?';

			$dc = ($dc == '' && $cookie <> date("j")) ? 'on' : '';
		
			if ($dc == 'on')
			{
				$calendar	= true;
				$d_action	= append_sid($url . 'dc=off');
				setcookie($this->config['cookie_name'] . '_calendar_on_header', 0,  time()+60*60*24, '/', $this->config['cookie_domain'], '');
			}

			if ($cookie == date("j"))
			{
				$calendar	= false;
				$d_action	= append_sid($url . 'dc=on');
			}

			if ($dc == 'off')
			{
				$calendar	= false;
				$d_action	= append_sid($url . 'dc=on');
				setcookie($this->config['cookie_name'] . '_calendar_on_header', date("j"),  time()+60*60*24, '/', $this->config['cookie_domain'], '');
			}
			$this->template->assign_vars(array(
				'DISPLAY_ACTION'		=> $d_action,
				'CALENDAR_ON_INDEX_OFF'	=> $this->config['hjw_calendar_on_index_off'],
			));
		}

		$this->template->assign_vars(array(
			'CALENDAR'				=> $calendar,
		));
		
		
		include($this->ext_root_path . 'includes/constants' . $this->phpex);
		$calendar_link	=	$this->helper->route('hjw_calendar_controller');

		if($this->config['version'] > '3.1')
		{
			$version = '3.1';
		}
		if($this->config['version'] > '3.2')
		{
			$version = '3.2';
		}
		
		$sql = 'SELECT *
				FROM ' . EXT_TABLE . '
				WHERE ext_name = "football/football"';
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$football = ($row['ext_active'] == 1) ? $football = $this->config['hjw_calendar_football'] : 0;
		
		$this->template->assign_vars(array(
			'U_CALENDAR'				=> $calendar_link,	
			'S_WEEK_OR_NEXT'			=> $this->config['hjw_calendar_week_or_next'],
			'S_WEEK_ON_INDEX'			=> $this->config['hjw_calendar_week_on_index'],
			'S_BIRTHDAY_ON_CALENDAR'	=> $this->config['hjw_calendar_birthday_on_calendar'],
			'S_CALENDAR_FOR_GUESTS'		=> $this->config['hjw_calendar_for_guests'],
			'VERSION'					=> $version,
		));	
		

		if ($calendar == true)
		{
			if($this->config['hjw_calendar_week_or_next'] == 1 || $this->config['hjw_calendar_week_or_next'] == 3)
			{
				if($this->user->data['user_timezone'])
				{
					date_default_timezone_set($this->user->data['user_timezone']);
				}
				else
				{
					date_default_timezone_set($this->config['board_timezone']);
				}
				$t_day   = $day   = date("j"); 
				$t_month = $month = date("n"); 
				$t_year  = $year  = date("Y"); 

				include($this->ext_root_path . 'includes/special_days' . $this->phpex);

				$c_days = ($this->config['hjw_calendar_number_of_weeks'] *7) -1;
				for ($y= 0; $y <= $c_days; $y++)
				{
					$hday = '';
					$i= $y;
					while ($i > 6)
					{
						$i -= 7;
					}
					$d = mktime(0, 0, 0, $t_month, $t_day+$y, $t_year);
					$day   = date("j", $d);
					$month = date("n", $d);
					$year  = date("Y", $d);

					include($this->ext_root_path . 'includes/special_day' . $this->phpex);

					$this->template->assign_block_vars('day', array(
						'INDEX'	=> true,
						'WD'	=> $this->user->lang['datetime'][date("l", $d)],
						'DATE'	=> date("d.", $d),
						'HDAY'	=> $hday,
						'BG'	=> strtolower(date("D", $d)),
						'I'		=> $i,
					));

					include($this->ext_root_path . 'includes/calendar_event' . $this->phpex);
				}
			}
		
			if($this->config['hjw_calendar_week_or_next'] == 2 || $this->config['hjw_calendar_week_or_next'] == 3)
			{
				$count_event_max = 10 *  $this->config['hjw_calendar_number_of_weeks'];
				$sql = 'SELECT *
						FROM ' . CALENDAR_TABLE . " 
						WHERE date_from >= '" . date("Y-m-d") . "' 
							OR 
								date_to >= '" . date("Y-m-d") . "'
									OR 
										date_from <= '" . date("Y-m-d") . "'
											AND 
												date_to = '0000-00-00' AND calendar_repeat = 1 
						ORDER by date_from";

				$event_result = $this->db->sql_query($sql);
				while ($event_row = $this->db->sql_fetchrow($event_result))
				{
					$hday		= '';
					$upcoming	= $event_row;
					$post_id	= $event_row['post_id'];
					$sql = 'SELECT *
							FROM ' . POSTS_TABLE . '
							WHERE post_id = ' . $post_id;
					$result = $this->db->sql_query($sql);
					while($row = $this->db->sql_fetchrow($result))
					{
						if ($row['post_visibility'] == 1)
						{
							$user_id = $this->user->data['user_id'];
							$auth_array = $this->auth->acl_raw_data($user_id, 'f_read', $row['forum_id']);
							if (isset($auth_array[$user_id][$row['forum_id']]['f_read']) && $auth_array[$user_id][$row['forum_id']]['f_read'])
							{
								if($event_row['calendar_repeat'])
								{
									$d_f = explode('-', $event_row['date_from']);
									$d_t = explode('-', $event_row['date_to']);
									$end = ($event_row['date_to'] != '0000-00-00') ? strtotime($event_row['date_to']) : mktime (0, 0, 0, 1, 18, 2038) ;
									$today = time();
									$count = 0;
									for($i= 0; $count < $count_event_max; $i++)
									{
										if($event_row['repeat_dm'] == 0)
										{
											$next = mktime(0, 0, 0, $d_f[1] + $i * $event_row['repeat_month_number'], $d_f[2], $d_f[0]);
										}
										if($event_row['repeat_dm'] == 1)
										{
											$next = mktime(0, 0, 0, $d_f[1], $d_f[2] + $i * $event_row['repeat_day_number'],$d_f[0]);
										}
										if($event_row['repeat_dm'] == 2)
										{
											$next = mktime(0, 0, 0, $d_f[1] + ($i * $event_row['repeat_month_number']), 1, $d_f[0]);
											$next_month = date("n", $next);
 											$next_year = date("Y", $next);
											$wd = date("N", $next);
											$next_day = 1 + $event_row['weekday'] - $wd + (($event_row['repeat_day_number']-1) * 7);
											if ($wd > $event_row['weekday'])
											{
												$next_day += 7;
											}
											
											if( checkdate($next_month, $next_day, $next_year))
											{
												$next = mktime (0, 0, 0, $next_month, $next_day, $next_year);
											}
											else
											{
												$next = $end + 1;
											}	
										}	
										if ($next > $end)
										{
											break;
										}
										if ($next >= $today)
										{
											$upcoming['date_from'] =  date("Y-m-d", $next);
											upcoming_events($upcoming);
											$count++;
										}
									}
								}
								else
								{		
									upcoming_events($upcoming);
								}
							}
						}
					}
				}
				$sql = 'SELECT *
						FROM ' . CALENDAR_EVENT_LIST_TABLE . "
						WHERE date_from >= '" . date("Y-m-d") . "'
							OR 
								date_to >= '" . date("Y-m-d") . "' 
						ORDER by date_from";
				$event_result = $this->db->sql_query($sql);
				while ($event_row = $this->db->sql_fetchrow($event_result))
				{
					$c_c++;
					$c_from[$c_c] = $event_row['date_from'];
//echo $c_to[$c_c] . '<br />';
					$c_to[$c_c] =  (($event_row['date_to'] <> '0000-00-00') && ($event_row['date_to'] > $event_row['date_from'])) ? $c_to[$c_c] = $event_row['date_to'] : '';
//echo $c_to[$c_c] . '<br />';
					$c_name[$c_c] = $event_row['appointment'];
					$c_title[$c_c] = $event_row['appointment'];
					if($event_row['description'] != '')
					{
						$c_title[$c_c]	.= ' - ' . $event_row['description'];
					}
					$c_link[$c_c] = $event_row['link'];
					$c_part[$c_c] = '';
					$c_nr[$c_c] = '';
					$c_color[$c_c] = $event_row['color'];
					$c_big[$c_c] = $event_row['big'];
					$c_bcolor[$c_c] = $event_row['bcolor'];
					$c_canceled[$c_c] = 0;
				}
				$sql = 'SELECT *
					FROM ' . CALENDAR_EVENT_LIST_TABLE . "
						WHERE date_from LIKE '0000-%'
							OR anniversary = 1";
				$event_result = $this->db->sql_query($sql);
				while ($event_row = $this->db->sql_fetchrow($event_result))
				{
					$r = explode('-', $event_row['date_from']);
					$from_year = str_pad($r[0], 4, '0', STR_PAD_LEFT);
					$from_month = str_pad($r[1], 2, '0', STR_PAD_LEFT);
					$from_day = str_pad($r[2], 2, '0', STR_PAD_LEFT);
					$r = explode('-', $event_row['date_to']);
					$to_year = str_pad($r[0], 4, '0', STR_PAD_LEFT);
					$to_month = str_pad($r[1], 2, '0', STR_PAD_LEFT);
					$to_day = str_pad($r[2], 2, '0', STR_PAD_LEFT);

					$from_day = (!date("L", mktime(0, 0, 0, 1, 1, date("Y"))) && $from_month == '02' && $from_day == '29') ? '28' : $from_day;

					if($from_year == '0000' || $event_row['anniversary'])
					{
						if($event_row['anniversary'])
						{
							$age = $from_year;
						}

						$from_year = date("Y");
						if ($from_month . $from_day < date("md"))
						{
							$from_year++;
						}
						
						$age = ($event_row['anniversary']) ? $event_row['appointment']	=	$event_row['appointment'] . ' (' . ($from_year - $age) . ')' : $age;

						if($to_year > '0000')
						{
							$to_year = $from_year - $r[0] + $to_year;
						}
					}
					$c_c++;
					$c_from[$c_c] = $from_year . '-' . $from_month . '-' . $from_day;
					$date_to = $to_year . '-' . $to_month . '-' . $to_day ;
					$c_to[$c_c] = ($date_to <> '0000-00-00' && $date_to > $event_row['date_from']) ? $to_year . '-' . $to_month . '-' . $to_day : '';
					$c_name[$c_c] = $event_row['appointment'];
					$c_title[$c_c] = $event_row['appointment'];
					if($event_row['description'])
					{
						$c_title[$c_c] .= ' - ' . $event_row['description'];
					}
					$c_link[$c_c] = $event_row['link'];
					$c_part[$c_c] = '';
					$c_nr[$c_c] = '';
					$c_color[$c_c] = $event_row['color'];
					$c_big[$c_c] = $event_row['big'];
					$c_bcolor[$c_c] = $event_row['bcolor'];
					$c_canceled[$c_c] = 0;
				}

				if($this->config['hjw_calendar_birthday_on_calendar'])
				{
					$sql = 'SELECT user_id, username, user_birthday, user_colour
							FROM ' . USERS_TABLE . '
							ORDER by user_birthday';

					$event_result = $this->db->sql_query($sql);
					while (($event_row = $this->db->sql_fetchrow($event_result)) && ($count_event < $count_event_max))
					{
						if($event_row['user_birthday'] && $event_row['user_birthday'] != ' 0- 0-   0')
						{
							$username = $event_row['username'];
							$birthday = $this->user->lang['BIRTHDAY'] . ' ' . $username;

							$r = explode('-', $event_row['user_birthday'] . '-00-00-00');
							$from_day = str_pad(trim($r[0]), 2, '0', STR_PAD_LEFT);
							$from_month = str_pad(trim($r[1]), 2, '0', STR_PAD_LEFT);
							$from_year = str_pad(trim($r[2]), 4, '0', STR_PAD_LEFT);
							$from_day = (!date("L", mktime(0, 0, 0, 1, 1, date("Y"))) && $from_month == '02' && $from_day == '29') ? '28' : $from_day;
							if (checkdate($from_month, $from_day, $from_year))
							{
								$user_age = $from_year;
								$from_year = date("Y");
								if($from_month . $from_day < date("md"))
								{
									$from_year++;
								}
								$user_age = $from_year - $user_age;
								if ($user_age >= 0)
								{
									$birthday = $user_age . '. ' . $birthday;
									$user_age = ' (' . $user_age . ')';
									$c_c++;
									$c_from[$c_c]		= $from_year . '-' . $from_month . '-' . $from_day ;
									$c_to[$c_c]			= '';
									$c_name[$c_c]		= $username . $user_age;
									$c_title[$c_c]		= $birthday;
									$c_link[$c_c]		= append_sid($this->phpbb_root_path . 'memberlist.php?mode=viewprofile&u=' . $event_row['user_id']);
									$c_part[$c_c]		= '';
									$c_nr[$c_c]			= '';
									$c_color[$c_c]		= $event_row['user_colour'];
									$c_bcolor[$c_c]		= '';
									$c_big[$c_c]		= '';
									$c_canceled[$c_c]	= 0;
								}
							}
						}
					}
				}

				if (isset($c_from))
				{
					array_multisort($c_from, $c_to, $c_name, $c_link, $c_part, $c_nr, $c_title, $c_color, $c_bcolor, $c_big, $c_canceled);
					$c_count = min(count($c_from), $count_event_max);
					for($i = 0; $i < $c_count; $i++)
					{
						$hday ='';
						$day   = date("j", strtotime($c_from[$i]));
						$month = date("n", strtotime($c_from[$i]));
						$year  = date("Y", strtotime($c_from[$i]));

						include($this->ext_root_path . 'includes/special_day' . $this->phpex);

						$this->template->assign_block_vars('event', array(
							'LINK'			=> $c_link[$i],
							'PART'			=> $c_part[$i],
							'NR'			=> $c_nr[$i],
							'EVENT_TITLE'	=> $c_title[$i],
							'TR'			=> chr(10) . str_repeat('-', strlen(utf8_decode($c_title[$i]))),
							'EVENT_NAME' 	=> $c_name[$i],
							'COLOR'			=> $c_color[$i],
							'BCOLOR'		=> $c_bcolor[$i],
							'BIG'			=> $c_big[$i],
							'WD'			=> (strpos(' ' . $user->data['user_dateformat'], 'D' ) > 0 || strpos(' ' . $user->data['user_dateformat'], 'l') > 0) ? '' : $this->user->lang['datetime'][date("l", strtotime($c_from[$i]))],
							'WD_TO'			=> (strpos(' ' . $user->data['user_dateformat'], 'D' ) > 0 || strpos(' ' . $user->data['user_dateformat'], 'l') > 0 || !$c_to[$i]) ? '' : $this->user->lang['datetime'][date("l", strtotime($c_to[$i]))],
							'DATE'			=> date_format ($c_from[$i], false),
							'DATE_TO'		=> ($c_to[$i] > $c_from[$i]) ? date_format ($c_to[$i], false) : '',
							'HDAY'			=> $hday,
							'BG'			=> strtolower(date("D", strtotime($c_from[$i]))),
							'BREAK'			=> (intval(($i+1)/10) == ($i+1)/10) ? true : false,
							'EVENTBG'		=> ($c_canceled[$i] == 0) ? 'eventbg' : 'no-eventbg',
						));
					}
					$this->template->assign_vars(array(
						'HD'			=> ($hday) ? true : false,
					));
				}
			}
		}
	}
	

	public function modify_participants_list()
	{
		$user_id  = $this->user->data['user_id'];

		include($this->ext_root_path . 'includes/constants' . $this->phpex);

		if ($this->request->variable('part', ''))
		{
			if ($user_id)
			{
				$pd	= $this->request->variable('d', '', true);
				$sql_ary = array(
					'POST_ID'		=> $this->request->variable('participants_id', '', true),
					'EVENT_DATE'	=> date("Y-m-d", $pd),
					'USER_ID'		=> $user_id,
					'NUMBER'		=> intval($this->request->variable('group', '', true)),
					'PARTICIPANTS'	=> utf8_normalize_nfc($this->request->variable('part', '', true)),
					'COMMENTS'		=> utf8_normalize_nfc($this->request->variable('comments', '', true)),
					'DATE'			=> date("Y-n-j-H-i"),
				);

				$vars = array(
					'sql_ary',
					'user_id',
				);
				
				extract($this->phpbb_dispatcher->trigger_event('hjw.calendar.viewtopic.modify_participants_list', compact($vars)));

				$sql = 'SELECT *
						FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
						WHERE post_id = "' . $sql_ary['POST_ID'] . '"
							AND event_date = "' . date("Y-m-d", $pd) . '"
							AND user_id = "' . $user_id . '" 
						OR post_id = "' . $sql_ary['POST_ID'] . '"
							AND event_date = ""
							AND user_id = "' . $user_id . '"'; 
				$result = $this->db->sql_query($sql);
				$row = $this->db->sql_fetchrow($result);
				if ($row)
				{
					$sql = 'UPDATE ' . CALENDAR_PARTICIPANTS_TABLE . '
							SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . '
							WHERE post_id = "' . $sql_ary['POST_ID'] . '"
								AND event_date = "' . date("Y-m-d", $pd) . '"
								AND user_id = "' . $user_id . '"
							OR post_id = "' . $sql_ary['POST_ID'] . '"
								AND event_date = ""
								AND user_id = "' . $user_id . '"'; 
					$notify = 'calendar_participants_change';
				}
				else
				{
					$sql = 'INSERT INTO ' . CALENDAR_PARTICIPANTS_TABLE . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);	
					$notify = 'calendar_participants';
				}
				$result = $this->db->sql_query($sql);
				if($this->config['hjw_calendar_notify_participating'] == true)
				{
					calendar_messenger($notify, $user_id, $sql_ary['POST_ID'], 'd=' . $pd . '&');
				}
			}
		}
	}

	public function calendar($event)
	{ 
		$mode = $this->request->variable('mode', '');
		if($mode != 'quote')
		{
			include($this->ext_root_path . 'includes/constants' . $this->phpex);
			$post_id = $event['post_id'];
			$forum_id = $event['forum_id'];
					
			$post = $event['page_data'];
			$post['SUBJECT'] = $this->request->variable('subject', $post['SUBJECT'], true);
			$post['MESSAGE'] = $this->request->variable('message', $post['MESSAGE'], true);
			$event['page_data'] = $post;

			$quote = ((substr($event['post_data']['post_text'],1,5) == 'quote')) ? true : false;

			$first_post_id = (isset($event['post_data']['topic_first_post_id'])) ? $event['post_data']['topic_first_post_id'] : 0;
			$first_post = ($first_post_id == 0) ? true : ($first_post_id == $event['post_id'] && !$quote) ? true : false;

			if (!($this->config['hjw_calendar_only_first_post'] && !$first_post))
			{
				$sql = 'SELECT *
						FROM ' . CALENDAR_FORUMS_TABLE . '
						WHERE forum_id = ' . $forum_id;
				$result = $this->db->sql_query($sql);
				$forum_row = $this->db->sql_fetchrow($result);
				$this->db->sql_freeresult($result);
				if ($forum_row)
				{
					if ($forum_row['allowed'] == 1)
					{
						$this->template->assign_vars( array(
							'CALENDAR_ALLOWED'			=> true,
						));	

						$month_name = array(
							1 => $this->user->lang['datetime']['January'],
							2 => $this->user->lang['datetime']['February'],
							3 => $this->user->lang['datetime']['March'],
							4 => $this->user->lang['datetime']['April'],
							5 => $this->user->lang['datetime']['May'],
							6 => $this->user->lang['datetime']['June'],
							7 => $this->user->lang['datetime']['July'],
							8 => $this->user->lang['datetime']['August'],
							9 => $this->user->lang['datetime']['September'],
						   10 => $this->user->lang['datetime']['October'],
						   11 => $this->user->lang['datetime']['November'],
						   12 => $this->user->lang['datetime']['December'],
						);
						$day_name = array(
							1 => $this->user->lang['datetime']['Monday'],
							2 => $this->user->lang['datetime']['Tuesday'],
							3 => $this->user->lang['datetime']['Wednesday'],
							4 => $this->user->lang['datetime']['Thursday'],
							5 => $this->user->lang['datetime']['Friday'],
							6 => $this->user->lang['datetime']['Saturday'],
							7 => $this->user->lang['datetime']['Sunday'],
						);

						$event_id = '';
						$event_name = '';
						$f_d = $t_d = array('0000', '00', '00');
						$calendar_repeat = 0;
						$calendar_repeat_dm = 0;
						$calendar_repeat_day_number = 7;
						$calendar_repeat_month_number = 1;
						$calendar_weekday = 1;
						$calendar_canceled = 0;
						if ($post_id)
						{
							$sql = 'SELECT *
									FROM ' . CALENDAR_TABLE . '
									WHERE post_id = ' . $post_id;
							$result = $this->db->sql_query($sql);
							$row = $this->db->sql_fetchrow($result);
							if($row)
							{
								$present = true;
								$event_id = $row['event_id'];
								$event_name = $row['event_name'];
								$f_d = explode('-', $row['date_from']);
								$t_d = explode('-', $row['date_to']);
							
								$calendar_repeat = $row['calendar_repeat'];
								$calendar_repeat_dm = $row['repeat_dm'];
								$calendar_repeat_day_number = $row['repeat_day_number'];
								$calendar_repeat_month_number = $row['repeat_month_number'];
								$calendar_weekday = $row['weekday'];
								$calendar_canceled = $row['canceled'];
							}
						}

						$event_id = $this->request->variable('event', $event_id);
						$event_name = utf8_normalize_nfc($this->request->variable('event_name', $event_name, true));
					
						$from_day = str_pad($this->request->variable('from_day', $f_d[2]), 2, '0', STR_PAD_LEFT);
						$from_month = str_pad($this->request->variable('from_month', $f_d[1]), 2, '0', STR_PAD_LEFT);
						$from_year = str_pad($this->request->variable('from_year', $f_d[0]), 4, '0', STR_PAD_LEFT);
						$to_day = str_pad($this->request->variable('to_day', $t_d[2]), 2, '0', STR_PAD_LEFT);
						$to_month = str_pad($this->request->variable('to_month', $t_d[1]), 2, '0', STR_PAD_LEFT);
						$to_year = str_pad($this->request->variable('to_year', $t_d[0]), 4, '0', STR_PAD_LEFT);
						$from = $from_year . '-' . $from_month . '-' . $from_day;
						
						$calendar_repeat = $this->request->variable('calendar_repeat', $calendar_repeat);
						$calendar_repeat_dm = $this->request->variable('calendar_repeat_dm', $calendar_repeat_dm);
						$calendar_repeat_day_number = $this->request->variable('calendar_repeat_day_number', $calendar_repeat_day_number);
						$calendar_repeat_month_number = $this->request->variable('calendar_repeat_month_number', $calendar_repeat_month_number);
						$calendar_repeat_day_name = $this->request->variable('calendar_every_weekday_name', $calendar_weekday);
						$calendar_canceled = $this->request->variable('calendar_canceled', $calendar_canceled);

						if($calendar_repeat_day_number < 1)
						{
							$calendar_repeat_day_number = 7;
						}
						if($calendar_repeat_month_number <1)
						{
							$calendar_repeat_month_number = 1;
						}

						$cr0 = (!$calendar_repeat) ? ' checked="checked" ' : '';
						$cr1 = ($calendar_repeat) ? ' checked="checked" ' : '';

						$crm  = ($calendar_repeat_dm == 0) ? ' checked="checked" ' : '';
						$crd  = ($calendar_repeat_dm == 1) ? ' checked="checked" ' : '';
						$crwd = '';
						if($calendar_repeat_dm == 2)
						{
							$crwd = ' checked="checked" ';
							$calendar_repeat_day_number	= $this->request->variable('calendar_every_weekday_number', $calendar_repeat_day_number);
							$calendar_repeat_day_name= $this->request->variable('calendar_every_weekday_name', $calendar_weekday);
							$calendar_repeat_month_number= $this->request->variable('calendar_every_month_number', $calendar_repeat_month_number);
						}
						$cc0 = ($calendar_canceled == 0) ? ' checked="checked" ' : '';
						$cc1 = ($calendar_canceled == 1) ? ' checked="checked" ' : '';
						
						$this->template->assign_vars(array(
							'CALENDAR_TAB'					=> $this->config['hjw_calendar_tab'],
							'EVENT_NAME' 					=> $event_name,
							'CR0'							=> $cr0,
							'CR1'							=> $cr1,
							'CRD'							=> $crd,
							'CALENDAR_REPEAT_DAY_NUMBER'	=> $calendar_repeat_day_number,
							'CRM'							=> $crm,
							'CALENDAR_REPEAT_MONTH_NUMBER'	=> $calendar_repeat_month_number,
							'CRWD'							=> $crwd,
							'CC0'							=> $cc0,
							'CC1'							=> $cc1,
							'ANNIVERSARY'					=> false,
						));	

						$this->template->assign_block_vars('from_day', array(
							'SELECT' =>'<option value=" ">' . $this->user->lang['DAY'] . '</option>',
						));	
						for ($i = 1; $i <= 31; $i++)
						{
							$s=($i == $from_day) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('from_day', array(
								'SELECT' =>'<option' . $s . ' value="' . $i . '">' . $i . '</option>',
							));	
						}

						$this->template->assign_block_vars('to_day', array(
							'SELECT' =>'<option value=" ">' . $this->user->lang['DAY'] . '</option>',
						));	
						for ($i = 1; $i <= 31; $i++)
						{
							$s = ($i == $to_day) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('to_day', array(
								'SELECT' =>'<option' . $s . ' value="' . $i . '">' . $i . '</option>',
							));	
						}
	
						$this->template->assign_block_vars('from_month', array(
							'SELECT' =>'<option value=" ">' . $this->user->lang['MONTH'] . '</option>',
						));	
						for ($i = 1; $i <= 12; $i++)
						{
							$s= ($i == $from_month) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('from_month', array(
								'SELECT' =>'<option' . $s . ' value="' . $i . '">' . $month_name[$i] . '</option>',
							));	
						}

						$this->template->assign_block_vars('to_month', array(
							'SELECT' =>'<option value=" ">' . $this->user->lang['MONTH'] . '</option>',
						));	
						for ($i = 1; $i <= 12; $i++)
						{
							$s = ($i == $to_month) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('to_month', array(
								'SELECT' =>'<option' . $s . ' value="' . $i . '">' . $month_name[$i] . '</option>',
							));	
						}

						$year = ($from_year > 0) ? $from_year : date("Y", time());
						$this->template->assign_block_vars('from_year', array(
							'SELECT' => '<option value=" ">' . $this->user->lang['YEAR'] . '</option>',
						));	
						for ($i = $year; $i < $year + 10; $i++)
						{
							$s = ($i == $from_year) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('from_year', array(
								'SELECT' => '<option' . $s . ' value="' . $i . '">' . $i . '</option>',
							));	
						}

						$this->template->assign_block_vars('to_year', array(
							'SELECT' => '<option value=" ">' . $this->user->lang['YEAR'] . '</option>',
						));	
						for ($i = $year; $i < $year + 10; $i++)
						{
							$s = ($i == $to_year) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('to_year', array(
								'SELECT' => '<option' . $s . ' value="' . $i . '">' . $i . '</option>',
							));	
						}

						for ($i = 1 ; $i < 6; $i++)
						{
							$s = ($i == $calendar_repeat_day_number) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('every', array(
								'SELECT' => '<option' . $s . ' value="' . $i . '">' . $i . '</option>',
							));	
						}

						for ($i = 1; $i < 8; $i++)
						{
							$s= ($i == $calendar_repeat_day_name) ? ' selected="selected"' : '';
							$this->template->assign_block_vars('weekday', array(
								'SELECT' => '<option' . $s . ' value="' . $i . '">' . $day_name[$i] . '</option>',
							));	
						}

						$this->template->assign_block_vars('event_select', array(
							'SELECT' => '<option style="color:#BBBBBB" value="0">' . $this->user->lang['CALENDAR_DELETE'] . '</option>',
						));	
						$sql = 'SELECT *
								FROM ' . CALENDAR_EVENT_TABLE . '
								ORDER by sort';
						$result = $this->db->sql_query($sql);
						while($row = $this->db->sql_fetchrow($result))
						{
							$s = (intval($row['id']) == intval($event_id)) ? ' selected="selected"' : '';
							if (intval($row['id']) == intval($event_id) && $row['anniversary'])
							{
								$this->template->assign_vars(array(
									'ANNIVERSARY'	=> true,
									'YEAR'			=> ($from_year) ? $from_year : date("Y"),
								));	
							}
							$this->template->assign_block_vars('event_select', array(
								'SELECT' => '<option' . $s . ' value="' . $row['id'] . '">' . $row['event'] . '</option>',
							));
						}
						
					}
				}
			}
		}
	}

	public function send_data_to_table($event)
	{
		$mode = $this->request->variable('mode','');
		if($mode != 'quote')
		{
			include($this->ext_root_path . 'includes/constants' . $this->phpex);
			$post_id = $event['data']['post_id'];

			$present  = false;
			$event_id = 0;
			$event_name = '';
			$f_d = $t_d = array('0000', '00', '00');
			$calendar_repeat = 0;
			$calendar_repeat_dm = 0;
			$calendar_repeat_day_number = 0;
			$calendar_repeat_month_number = 0;
			$calendar_weekday = 1;
			$calendar_canceled = 0;
			
			$sql = 'SELECT *
					FROM ' . CALENDAR_TABLE . '
					WHERE post_id = ' . $post_id;
			$result = $this->db->sql_query($sql);
			if($row = $this->db->sql_fetchrow($result))
			{
				$present = true;
				$event_id = $row['event_id'];
				$event_name = $row['event_name'];
				$f_d = explode('-',$row['date_from']);
				$t_d = explode('-',$row['date_to']);

				$calendar_repeat = $row['calendar_repeat'];
				$calendar_repeat_dm = $row['repeat_dm'];
				$calendar_repeat_day_number = $row['repeat_day_number'];
				$calendar_repeat_month_number = $row['repeat_month_number'];
				$calendar_weekday = $row['weekday'];
				$calendar_canceled = $row['canceled'];
			}
			
			$event_id = $this->request->variable('event', $event_id);
			$event_name = utf8_normalize_nfc($this->request->variable('event_name', $event_name, true));
			$from_day = str_pad($this->request->variable('from_day', $f_d[2]), 2, '0', STR_PAD_LEFT);
			$from_month = str_pad($this->request->variable('from_month', $f_d[1]), 2, '0', STR_PAD_LEFT);
			$from_year = str_pad($this->request->variable('from_year', $f_d[0]), 4, '0', STR_PAD_LEFT);
			$to_day = str_pad($this->request->variable('to_day', $t_d[2]), 2, '0', STR_PAD_LEFT);
			$to_month = str_pad($this->request->variable('to_month', $t_d[1]), 2, '0', STR_PAD_LEFT);
			$to_year = str_pad($this->request->variable('to_year', $t_d[0]), 4, '0', STR_PAD_LEFT);
			$from = $from_year . '-' . $from_month . '-' . $from_day;
			$to = $to_year . '-' . $to_month . '-' . $to_day;

			$calendar_repeat				= $this->request->variable('calendar_repeat', $calendar_repeat);
			$calendar_repeat_dm				= $this->request->variable('calendar_repeat_dm', $calendar_repeat_dm);
			$calendar_repeat_day_number		= $this->request->variable('calendar_repeat_day_number', $calendar_repeat_day_number);
			$calendar_repeat_month_number	= $this->request->variable('calendar_repeat_month_number', $calendar_repeat_month_number);
			$calendar_weekday				= $this->request->variable('calendar_every_weekday_name', $calendar_weekday);
			$calendar_canceled				= $this->request->variable('calendar_canceled', $calendar_canceled);
			$sql = 'SELECT *
					FROM ' . CALENDAR_EVENT_TABLE . '
					WHERE id = ' . $event_id;
			$event_result = $this->db->sql_query($sql);
			$event_row = $this->db->sql_fetchrow($event_result);
			$calendar_anniversary = $event_row['anniversary'];
			if($calendar_repeat_day_number == 0)
			{
				$calendar_repeat_day_number = 7;
			}
			if($calendar_repeat_month_number == 0)
			{
				$calendar_repeat_month_number = 1;
			}
			if($calendar_repeat_dm == 2)
			{
				$calendar_repeat_day_number		= $this->request->variable('calendar_every_weekday_number', $calendar_repeat_day_number);
				$calendar_repeat_month_number	= $this->request->variable('calendar_every_month_number', $calendar_repeat_month_number);
			}

			$sql_ary = array(
				'POST_ID'				=>	$post_id,
				'EVENT_ID'				=>	$event_id,
				'EVENT_NAME'			=>	$event_name,
				'DATE_FROM'				=>	$from,
				'DATE_TO'				=>	$to,
				'CALENDAR_REPEAT'		=>	$calendar_repeat,
				'REPEAT_DM'				=>	$calendar_repeat_dm,
				'REPEAT_DAY_NUMBER'		=>	$calendar_repeat_day_number,
				'REPEAT_MONTH_NUMBER'	=>	$calendar_repeat_month_number,
				'WEEKDAY'				=>	$calendar_weekday,
				'CANCELED'				=>	$calendar_canceled,
				'ANNIVERSARY'			=>	$calendar_anniversary,
			);
	
			if ($present & $event_id == 0)
			{
				$sql = 'DELETE FROM ' . CALENDAR_TABLE . ' 
					WHERE post_id = ' . $post_id; 
				$result = $this->db->sql_query($sql);
			}

			if ($event_id > 0)
			{
				if ($present)
				{
					// Calendar-Entry change
					$sql = 'Select *
							FROM ' . CALENDAR_TABLE . '
							WHERE post_id = ' . $post_id;
					$old_result = $this->db->sql_query($sql);
					$old_row = $this->db->sql_fetchrow($old_result);
					$old_ary = array(
						'POST_ID'				=>	$old_row['post_id'],
						'EVENT_ID'				=>	$old_row['event_id'],
						'EVENT_NAME'			=>	$old_row['event_name'],
						'DATE_FROM'				=>	$old_row['date_from'],
						'DATE_TO'				=>	$old_row['date_to'],
						'CALENDAR_REPEAT'		=>	$old_row['calendar_repeat'],
						'REPEAT_DM'				=>	$old_row['repeat_dm'],
						'REPEAT_DAY_NUMBER'		=>	$old_row['repeat_day_number'],
						'REPEAT_MONTH_NUMBER'	=>	$old_row['repeat_month_number'],
						'WEEKDAY'				=>	$old_row['weekday'],
						'CANCELED'				=>	$old_row['canceled'],
						'ANNIVERSARY'			=>	$old_row['anniversary'],
					);
					if($old_ary != $sql_ary)
					{			
						$sql = 'UPDATE ' . CALENDAR_TABLE . '
							SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . '
								WHERE post_id = ' . $post_id;
						$result = $this->db->sql_query($sql);
						$notify = 'calendar_notify_change';
					}
					else
					{
						$notify = false; 
					}
				}
				else
				{
					// New Calendar-Entry
					$sql = 'INSERT INTO ' . CALENDAR_TABLE . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);	
					$result = $this->db->sql_query($sql);
					$notify = 'calendar_notify';
				}
				if($notify && $this->config['hjw_calendar_notify'] == true)
				{
					calendar_messenger($notify, $event['data']['poster_id'], $post_id, '');
				}
			}
		}
	}
}

function calendar_messenger($mode, $author_id, $post_id, $pd)
{
	global $user, $db, $phpbb_root_path, $phpEx, $config;
	include_once($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);

	$sql = 'SELECT poster_id, forum_id, post_subject
			FROM ' . POSTS_TABLE . '
			WHERE post_id = ' . $post_id;
	$post_result = $db->sql_query($sql);
	$post_row = $db->sql_fetchrow($post_result);

	$sql = 'SELECT username
		FROM ' . USERS_TABLE . '
			WHERE user_id = ' . $author_id;
	$user_result = $db->sql_query($sql);
	$user_row = $db->sql_fetchrow($user_result);
	$author = $user_row['username'];
	$part_user = array();
	$poster = false;

	if(($mode == 'calendar_participants' || $mode == 'calendar_participants_change') && $config['hjw_calendar_notify_participating'] == true && $config['hjw_calendar_notify_participating_adress'] != 1)
	{
		if($config['hjw_calendar_notify_participating_adress'] != 2)
		{
			$sql = 'SELECT user_id
					FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
					WHERE post_id = ' . $post_id;
			$part_result = $db->sql_query($sql);
			while($part_row = $db->sql_fetchrow($part_result))
			{
				$part_user[] = $part_row['user_id'];
				if($part_row['user_id'] == $post_row['poster_id'])
				{
					$poster = true;
				}
			}
		}
		
		if($poster == false)
		{
			$part_user[] = $post_row['poster_id'];
		}

		for($i = 0; $i < count($part_user); $i++)
		{

			$sql = 'SELECT username, user_email, user_lang, user_inactive_reason
					FROM ' . USERS_TABLE . '
					WHERE user_id = ' . $part_user[$i];
		
			$user_result = $db->sql_query($sql);
			$user_row = $db->sql_fetchrow($user_result);
			
			calendar_messenger_send($mode, $user_row['user_lang'], $user_row['user_email'], $config['sitename'], $author, $user_row['username'], $user_row['user_inactive_reason'], $post_row['post_subject'], $post_row['forum_id'], $post_id, $part_user[$i], $author_id, $pd);
		}
	}
	else
	{
		$sql = 'SELECT user_id, username, user_email, user_lang, user_inactive_reason
				FROM ' . USERS_TABLE;
		
		$user_result = $db->sql_query($sql);
		while($user_row = $db->sql_fetchrow($user_result))
		{
			calendar_messenger_send($mode, $user_row['user_lang'], $user_row['user_email'], $config['sitename'], $author, $user_row['username'], $user_row['user_inactive_reason'], $post_row['post_subject'], $post_row['forum_id'], $post_id, $user_row['user_id'] ,$author_id, $pd);
		}
	}
}

function calendar_messenger_send($mode, $lang, $email, $sitename, $author, $username, $inactive, $subject, $forum_id, $post_id, $user_id, $author_id, $pd)
{
	global $phpbb_root_path, $auth, $db;
	$url = generate_board_url();

	$sql = 'SELECT ban_userid
			FROM ' . BANLIST_TABLE . '
			WHERE ban_userid = ' . $user_id;
	$ban_result = $db->sql_query($sql);
	$ban_row = $db->sql_fetchrow($ban_result);
	if(!$ban_row)
	{
		if($username && $email && $lang && !$inactive && $user_id != $author_id)
		{
			$auth_array = $auth->acl_raw_data($user_id, 'f_read', $forum_id);
			if(isset($auth_array[$user_id][$forum_id]['f_read']) && $auth_array[$user_id][$forum_id]['f_read'])
			{
				$mail_template_path = $phpbb_root_path . 'ext/hjw/calendar/language/' . $lang . '/email/';

				$messenger = new \messenger();
				$messenger->template($mode, $lang , $mail_template_path);
				$messenger->to($email, $sitename);
				$messenger->assign_vars(array(
					'AUTHOR'   			=> $author,
					'USERNAME'   		=> $username,
					'EVENT_NAME' 		=> $subject,
					'U_CALENDAR_EVENT' 	=> $url . '/viewtopic.php?' . $pd . 'p=' . $post_id,
				));
				$messenger->send();
			}
		}
	}
}

function upcoming_events($upcoming)
{
	global $db, $user, $auth, $phpbb_root_path, $config, $wday, $c_event, $c_c, $c_from, $c_to, $c_name, $c_link, $c_part, $c_nr, $c_title, $c_color, $c_bcolor, $c_big, $c_canceled;

	$sql = 'SELECT *
			FROM ' . CALENDAR_EVENT_TABLE . '
			WHERE id = ' . $upcoming['event_id'];
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	
	$sql = 'SELECT *
			FROM ' . POSTS_TABLE . '
			WHERE post_id = ' . $upcoming['post_id']; 
	$post_result = $db->sql_query($sql);
	$post_row = $db->sql_fetchrow($post_result);
	$db->sql_freeresult($post_result);
	if ($post_row)
	{
		if ($post_row['post_visibility'] == 1)
		{
			$auth_array = $auth->acl_raw_data($user->data['user_id'], 'f_read', $post_row['forum_id']);
			if (isset($auth_array[$user->data['user_id']][$post_row['forum_id']]['f_read']) && $auth_array[$user->data['user_id']][$post_row['forum_id']]['f_read'])
			{	
				$bdate = '';
				$event_date = $upcoming['date_from'];
				if ($upcoming['calendar_repeat'])
				{
					$bdate = 'd=' . strtotime(($upcoming['date_from'])) . '&';
					$upcoming['date_to'] = '0000-00-00';
				}
				$link 	= append_sid($phpbb_root_path . 'viewtopic.php?' . $bdate . 'p=' . $post_row['post_id'] . '#p' . $post_row['post_id']);
				$number = 0;
				$nr = '';
				$part='';

				if($config['hjw_calendar_number_participating'])
				{
					$sql = 'SELECT *
							FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
							WHERE post_id = "' . $upcoming['post_id'] . '"
								AND event_date = "' . $event_date  . '"
							OR post_id = "' . $upcoming['post_id'] . '"
								AND event_date = ""';
					$part_result = $db->sql_query($sql);
					while($part_row = $db->sql_fetchrow($part_result))
					{
						$sql = 'SELECT user_colour, username
								FROM ' . USERS_TABLE . '
								WHERE user_id = ' . $part_row['user_id'];
						$user_result = $db->sql_query($sql);
						$user_row = $db->sql_fetchrow($user_result);
	
						if ($part_row['participants'] == 'yes')
						{
							$number += intval($part_row['number']);
							if($config['hjw_calendar_participants_name'])
							{
								$part .= chr(10) . $user_row['username'] . ' [' . intval($part_row['number']) . ']';
							}
						}
					}
				}
				$c_c++;
				$c_from[$c_c] = date("Y-m-d", strtotime($upcoming['date_from']));
				$c_to[$c_c] = ($upcoming['date_to'] && $upcoming['date_to'] <> '0000-00-00') ? date("Y-m-d", strtotime($upcoming['date_to'])) : '';
				$c_name[$c_c] = $upcoming['event_name'];
				$c_title[$c_c] = $post_row['post_subject']; 
				$c_part[$c_c] = $part;
				$c_nr[$c_c] = $number;
				$c_link[$c_c] = $link;
				$c_color[$c_c] = $row['color'];
				$c_bcolor[$c_c] = $row['bcolor'];
				$c_big[$c_c] = $row['big'];
				$c_canceled[$c_c] = $upcoming['canceled'];
			}
		}
	}
	else
	{
		$sql = 'DELETE FROM ' . CALENDAR_TABLE . ' 
				WHERE post_id = ' . $upcoming['post_id']; 
		$result = $db->sql_query($sql);
	}
}
function date_format ($event_date, $time)
{
	global $user, $config;
	
	$d_f	= explode('-',$event_date . '-0-0-0');
	$d = mktime($d_f[3], $d_f[4], $d_f[5], $d_f[1], $d_f[2], $d_f[0]);
	date_default_timezone_set($user->data['user_timezone']);
	$r	= explode('-',date("Y-m-j-H-i-s", $d));
	date_default_timezone_set($config['board_timezone']);
	if($time)
	{
		$date_format = $user->data['user_dateformat'];
	}
	else
	{
		$date_format = trim(preg_replace(array('/a/','/A/','/B/','/g/','/G/','/h/','/H/','/i/','/s/','/u/','/:/'),'',$user->data['user_dateformat']),' ,');
	}
	return $user->create_datetime()
		->setDate($r[0],$r[1],$r[2])
		->setTime($r[3],$r[4],$r[5])
		->format($date_format, true);
}